#!/usr/bin/env python

#############################################################################
##
## Copyright (C) 2004-2005 Trolltech AS. All rights reserved.
##
## This file is part of the example classes of the Qt Toolkit.
##
## This file may be used under the terms of the GNU General Public
## License version 2.0 as published by the Free Software Foundation
## and appearing in the file LICENSE.GPL included in the packaging of
## this file.  Please review the following information to ensure GNU
## General Public Licensing requirements will be met:
## http://www.trolltech.com/products/qt/opensource.html
##
## If you are unsure which license is appropriate for your use, please
## review the following information:
## http://www.trolltech.com/products/qt/licensing.html or contact the
## sales department at sales@trolltech.com.
##
## This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
## WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
##
#############################################################################

import sys
from PyQt4 import QtCore, QtGui


class TabDialog(QtGui.QDialog):
    def __init__(self, fileName, parent=None):
        QtGui.QDialog.__init__(self, parent)
        
        fileInfo = QtCore.QFileInfo(fileName)

        tabWidget = QtGui.QTabWidget()
        tabWidget.addTab(GeneralTab(fileInfo), self.tr("General"))
        tabWidget.addTab(PermissionsTab(fileInfo), self.tr("Permissions"))
        tabWidget.addTab(ApplicationsTab(fileInfo), self.tr("Applications"))
    
        okButton = QtGui.QPushButton(self.tr("OK"))
        cancelButton = QtGui.QPushButton(self.tr("Cancel"))
    
        self.connect(okButton, QtCore.SIGNAL("clicked()"), self, QtCore.SLOT("accept()"))
        self.connect(cancelButton, QtCore.SIGNAL("clicked()"), self, QtCore.SLOT("reject()"))
    
        buttonLayout = QtGui.QHBoxLayout()
        buttonLayout.addStretch(1)
        buttonLayout.addWidget(okButton)
        buttonLayout.addWidget(cancelButton)
    
        mainLayout = QtGui.QVBoxLayout()
        mainLayout.addWidget(tabWidget)
        mainLayout.addLayout(buttonLayout)
        self.setLayout(mainLayout)
    
        self.setWindowTitle(self.tr("Tab Dialog"))


class GeneralTab(QtGui.QWidget):
    def __init__(self, fileInfo, parent=None):
        QtGui.QWidget.__init__(self, parent)
        
        fileNameLabel = QtGui.QLabel(self.tr("File Name:"))
        fileNameEdit = QtGui.QLineEdit(fileInfo.fileName())
    
        pathLabel = QtGui.QLabel(self.tr("Path:"))
        pathValueLabel = QtGui.QLabel(fileInfo.absoluteFilePath())
        pathValueLabel.setFrameStyle(QtGui.QFrame.Panel | QtGui.QFrame.Sunken)
    
        sizeLabel = QtGui.QLabel(self.tr("Size:"))
        size = fileInfo.size() / 1024
        sizeValueLabel = QtGui.QLabel(self.tr("%1 K").arg(size))
        sizeValueLabel.setFrameStyle(QtGui.QFrame.Panel | QtGui.QFrame.Sunken)
    
        lastReadLabel = QtGui.QLabel(self.tr("Last Read:"))
        lastReadValueLabel = QtGui.QLabel(fileInfo.lastRead().toString())
        lastReadValueLabel.setFrameStyle(QtGui.QFrame.Panel | QtGui.QFrame.Sunken)
    
        lastModLabel = QtGui.QLabel(self.tr("Last Modified:"))
        lastModValueLabel = QtGui.QLabel(fileInfo.lastModified().toString())
        lastModValueLabel.setFrameStyle(QtGui.QFrame.Panel | QtGui.QFrame.Sunken)
    
        mainLayout = QtGui.QVBoxLayout()
        mainLayout.addWidget(fileNameLabel)
        mainLayout.addWidget(fileNameEdit)
        mainLayout.addWidget(pathLabel)
        mainLayout.addWidget(pathValueLabel)
        mainLayout.addWidget(sizeLabel)
        mainLayout.addWidget(sizeValueLabel)
        mainLayout.addWidget(lastReadLabel)
        mainLayout.addWidget(lastReadValueLabel)
        mainLayout.addWidget(lastModLabel)
        mainLayout.addWidget(lastModValueLabel)
        mainLayout.addStretch(1)
        self.setLayout(mainLayout)


class PermissionsTab(QtGui.QWidget):
    def __init__(self, fileInfo, parent=None):
        QtGui.QWidget.__init__(self, parent)

        permissionsGroup = QtGui.QGroupBox(self.tr("Permissions"))
    
        readable = QtGui.QCheckBox(self.tr("Readable"))
        if fileInfo.isReadable():
            readable.setChecked(True)
    
        writable = QtGui.QCheckBox(self.tr("Writable"))
        if fileInfo.isWritable():
            writable.setChecked(True)
    
        executable = QtGui.QCheckBox(self.tr("Executable"))
        if fileInfo.isExecutable():
            executable.setChecked(True)
    
        ownerGroup = QtGui.QGroupBox(self.tr("Ownership"))
    
        ownerLabel = QtGui.QLabel(self.tr("Owner"))
        ownerValueLabel = QtGui.QLabel(fileInfo.owner())
        ownerValueLabel.setFrameStyle(QtGui.QFrame.Panel | QtGui.QFrame.Sunken)
    
        groupLabel = QtGui.QLabel(self.tr("Group"))
        groupValueLabel = QtGui.QLabel(fileInfo.group())
        groupValueLabel.setFrameStyle(QtGui.QFrame.Panel | QtGui.QFrame.Sunken)
    
        permissionsLayout = QtGui.QVBoxLayout()
        permissionsLayout.addWidget(readable)
        permissionsLayout.addWidget(writable)
        permissionsLayout.addWidget(executable)
        permissionsGroup.setLayout(permissionsLayout)
    
        ownerLayout = QtGui.QVBoxLayout()
        ownerLayout.addWidget(ownerLabel)
        ownerLayout.addWidget(ownerValueLabel)
        ownerLayout.addWidget(groupLabel)
        ownerLayout.addWidget(groupValueLabel)
        ownerGroup.setLayout(ownerLayout)
    
        mainLayout = QtGui.QVBoxLayout()
        mainLayout.addWidget(permissionsGroup)
        mainLayout.addWidget(ownerGroup)
        mainLayout.addStretch(1)
        self.setLayout(mainLayout)


class ApplicationsTab(QtGui.QWidget):
    def __init__(self, fileInfo, parent=None):
        QtGui.QWidget.__init__(self, parent)
        
        topLabel = QtGui.QLabel(self.tr("Open with:"))
    
        applicationsListBox = QtGui.QListWidget()
        applications = QtCore.QStringList()
    
        for i in range(1, 31):
            applications.append(self.tr("Application %1").arg(i))
        
        applicationsListBox.insertItems(0, applications)
    
        alwaysCheckBox = QtGui.QCheckBox()
    
        if fileInfo.suffix().isEmpty():
            alwaysCheckBox = QtGui.QCheckBox(self.tr("Always use this application to open this type of file"))
        else:
            alwaysCheckBox = QtGui.QCheckBox(self.tr("Always use this application to open files with the extension '%1'").arg(fileInfo.suffix()))
    
        layout = QtGui.QVBoxLayout()
        layout.addWidget(topLabel)
        layout.addWidget(applicationsListBox)
        layout.addWidget(alwaysCheckBox)
        self.setLayout(layout)


if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    fileName = QtCore.QString()

    if len(sys.argv) >= 2:
        fileName = sys.argv[1]
    else:
        fileName = "."

    tabdialog = TabDialog(fileName)
    sys.exit(tabdialog.exec_())
